﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Linq;
using System.Windows.Forms;
using UrzaIncubator;
using System.Reflection;
using System.IO;
using Color = System.Drawing.Color;

namespace UrzaBackgroundPictures
{
    static class Program
    {
        static NotifyIcon icon;
        static System.Threading.Timer timer;
        static string currentExpansionFilter;
        static string currentBlockFilter;

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            try
            {
                icon = new NotifyIcon { Visible = true, ContextMenuStrip = new ContextMenuStrip() };

                DataRoot.WorkingPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), "UrzaData");

                Tools.TestConnection();

                using (Entities entities = Entities.Create())
                {
                    // Blocks
                    foreach (Block block in entities.Blocks.OrderByDescending(b => b.Index))
                    {
                        if (block.Name == "Oracle")
                            continue;
                        Image img = null;
                        if (block.Image != null)
                        {
                            using (MemoryStream ms = new MemoryStream(block.Image))
                            {
                                img = GetARGBImage(ms);
                            }
                        }

                        ToolStripMenuItem rootItem = new ToolStripMenuItem(block.Name, img, CheckRoot);
                        icon.ContextMenuStrip.Items.Add(rootItem);

                        if (block.Complete)
                        {
                            rootItem.ForeColor = Color.DarkGreen;
                        }
                        else
                        {
                            rootItem.Font = new Font(rootItem.Font, FontStyle.Italic);
                        }

                        foreach (Expansion exp in block.Expansions.OrderBy(e => e.OrderInBlock))
                        {
                            Image expImage = null;
                            if (exp.Image != null)
                            {
                                using (MemoryStream ms = new MemoryStream(exp.Image))
                                {
                                    expImage = GetARGBImage(ms);
                                }
                            }
                            ToolStripMenuItem child = new ToolStripMenuItem(exp.Name, expImage, CheckNode);
                            rootItem.DropDownItems.Add(child);

                            if (exp.Complete)
                            {
                                child.Font = new Font(child.Font, FontStyle.Regular);
                                child.ForeColor = Color.DarkGreen;
                            }
                            else
                            {
                                child.Font = new Font(child.Font, FontStyle.Italic);
                            }
                        }
                    }
                }
                // Controls
                icon.ContextMenuStrip.Items.Add("-");
                ToolStripMenuItem item = new ToolStripMenuItem("Next", null, Next);
                icon.ContextMenuStrip.Items.Add(item);
                icon.ContextMenuStrip.Items.Add("-");
                item = new ToolStripMenuItem("Close", Image.FromStream(Assembly.GetExecutingAssembly().GetManifestResourceStream("UrzaBackgroundPictures.close.png")), Quit);
                icon.ContextMenuStrip.Items.Add(item);

                icon.Icon = new Icon(Assembly.GetExecutingAssembly().GetManifestResourceStream("UrzaBackgroundPictures.MTG Green.ico"));
                icon.Text = @"Urza Background Pictures";

                timer = new System.Threading.Timer(
                    o =>
                    {
                        using (Entities entities = Entities.Create())
                        {
                            Tools.GenerateDesktopBackground(entities, currentBlockFilter, currentExpansionFilter);
                        }
                    }, null, 0, 60000);

                Application.Run();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        static Image GetARGBImage(MemoryStream ms)
        {
            const int size = 24;
            Image img;
            using (Image src = Image.FromStream(ms))
            {
                img = new Bitmap(size, size, PixelFormat.Format32bppArgb);
                using (Graphics gfx = Graphics.FromImage(img))
                {
                    int width;
                    int height;

                    if (src.Width > src.Height)
                    {
                        float ratio = (float)size / src.Width;
                        width = size;
                        height = (int)(src.Height * ratio);
                    }
                    else
                    {
                        float ratio = (float)size / src.Height;
                        height = size;
                        width = (int)(src.Width * ratio);
                    }

                    gfx.DrawImage(src, Math.Abs(width - size) / 2, Math.Abs(height - size) / 2, width, height);
                }
            }
            return img;
        }

        static void UncheckItem(ToolStripMenuItem item)
        {
            item.Checked = false;
            foreach (ToolStripMenuItem child in item.DropDownItems)
            {
                UncheckItem(child);
            }
        }

        static void CheckRoot(object sender, EventArgs e)
        {
            ToolStripMenuItem currentItem = sender as ToolStripMenuItem;
            bool currentState = currentItem.Checked;

            foreach (ToolStripItem item in icon.ContextMenuStrip.Items)
            {
                ToolStripMenuItem menu = item as ToolStripMenuItem;
                if (menu != null)
                    UncheckItem(menu);
            }

            currentExpansionFilter = null;

            if (currentState)
            {
                currentBlockFilter = null;
            }
            else
            {
                currentItem.Checked = true;
                currentBlockFilter = currentItem.Text;
            }
        }

        static void CheckNode(object sender, EventArgs e)
        {
            ToolStripMenuItem currentItem = sender as ToolStripMenuItem;
            bool currentState = currentItem.Checked;

            foreach (ToolStripItem item in icon.ContextMenuStrip.Items)
            {
                ToolStripMenuItem menu = item as ToolStripMenuItem;
                if (menu != null)
                    UncheckItem(menu);
            }

            if (currentState)
            {
                currentExpansionFilter = null;
            }
            else
            {
                currentItem.Checked = true;
                (currentItem.OwnerItem as ToolStripMenuItem).Checked = true;
                currentExpansionFilter = currentItem.Text;
            }
        }

        static void Next(object sender, EventArgs e)
        {
            timer.Change(0, 60000);
        }

        static void Quit(object sender, EventArgs e)
        {
            timer.Dispose();
            icon.Visible = false;
            icon.Dispose();
            Application.Exit();
        }

    }
}
