﻿using System;
using System.Linq;
using System.Threading.Tasks;
using UrzaIncubator;

namespace UrzaGatherer.ViewModels
{
    public class BlockViewModel : ViewModelBase
    {
        Block current;
        IOrderedQueryable<Block> blocks;
        readonly RelayCommand addBlockCommand;
        readonly RelayCommand removeBlockCommand;
        readonly RelayCommand removeExpansionCommand;
        readonly RelayCommand rescanCommand;
        readonly RootViewModel rootViewModel;

        public RelayCommand AddBlockCommand
        {
            get { return addBlockCommand; }
        }

        public RelayCommand RemoveBlockCommand
        {
            get { return removeBlockCommand; }
        }

        public RelayCommand RemoveExpansionCommand
        {
            get { return removeExpansionCommand; }
        }

        public RelayCommand RescanCommand
        {
            get { return rescanCommand; }
        }

        public Block Current
        {
            get { return current; }
            set
            {
                if (current == value)
                    return;

                current = value;

                rootViewModel.ExpansionViewModel.Refresh();
            }
        }

        public BlockViewModel(RootViewModel rootViewModel)
        {
            this.rootViewModel = rootViewModel;
            Refresh();
            addBlockCommand = new RelayCommand(o => AddBlock());
            removeBlockCommand = new RelayCommand(o => RemoveBlock());
            removeExpansionCommand = new RelayCommand(o => RemoveExpansion());
            rescanCommand = new RelayCommand(o => Rescan());
        }

        public IOrderedQueryable<Block> Blocks
        {
            get { return blocks; }
        }

        public void RecomputeCounters()
        {
            Expansion expansion = rootViewModel.ExpansionViewModel.Current;
            expansion.Complete = expansion.Cards.All(card => card.Check);

            Block block = expansion.Block;
            block.Complete = block.Expansions.All(exp => exp.Complete);

            rootViewModel.ExpansionViewModel.Refresh();
        }

        void RemoveExpansion()
        {
            if (RootViewModel.DialogSystem.Question("Are you sure you want to delete this expansion?", "Confirmation"))
            {
                Block block = rootViewModel.ExpansionViewModel.Current.Block;
                DataRoot.Entities.DeleteObject(rootViewModel.ExpansionViewModel.Current);

                rootViewModel.Commit();

                block.RefreshSortedExpansions();
            }
        }

        void RemoveBlock()
        {
            if (RootViewModel.DialogSystem.Question("Are you sure you want to delete this block?", "Confirmation"))
            {
                DataRoot.Entities.DeleteObject(Current);

                rootViewModel.Commit();

                Refresh();
            }
        }

        void AddBlock()
        {
            string result = RootViewModel.DialogSystem.Query("Block Name:", "Add a new block");

            if (result != null)
            {
                Block block = new Block
                {
                    Name = result,
                    Index = (from b in DataRoot.Entities.Blocks select b.Index).Max() + 1,
                    ID = (from b in DataRoot.Entities.Blocks select b.ID).Max() + 1,
                };

                DataRoot.Entities.AddToBlocks(block);
                rootViewModel.Commit();

                Refresh();
            }
        }

        public void Refresh()
        {
            blocks = DataRoot.Entities.Blocks.OrderByDescending(b => b.Index);

            RaisePropertyChanged(()=>Blocks);
        }

        void Rescan()
        {
            if (!RootViewModel.DialogSystem.Question("Are you sure you want to rescan all cards from gatherer.wizards.com?\nThe process can take several hours.", "Confirmation")) return;

            RootViewModel.DialogSystem.ShowWaitCursor(true);
            Task task = Task.Factory.StartNew(() =>
            {
                try
                {
                    Card[] cards = DataRoot.Entities.Cards.ToArray();

                    foreach (Card card in cards)
                    {
                        Tools.GetCardInfos(card);
                        rootViewModel.Commit();
                    }
                    RootViewModel.DialogSystem.Message("Operation completed.", "UrzaGatherer");
                }
                catch (Exception ex)
                {
                    RootViewModel.DialogSystem.Message(ex.Message, "Error during scan process");
                }
            });
            task.Wait();
            RootViewModel.DialogSystem.ShowWaitCursor(false);
        }

        public Expansion CreateExpansion(string expansionName)
        {
            Expansion expansion = new Expansion
                                      {
                                          ID = (from e in DataRoot.Entities.Expansions select e.ID).Max() + 1, 
                                          Name = expansionName,
                                          Block = Current,
                                          FrenchVersion = true
                                      };

            Current.Expansions.Add(expansion);

            DataRoot.Entities.AddToExpansions(expansion);

            return expansion;
        }
    }
}
