﻿using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using UrzaIncubator;

namespace UrzaGatherer.ViewModels
{
    public class ChartViewModel : ViewModelBase
    {
        readonly RootViewModel rootViewModel;
        List<DataChart> colorsChart;
        List<DataChart> rarityChart;

        int currentChecked;
        int currentMissing;
        float currentValueCount;
        float totalValueCount;

        int expansionsCount;
        int blocksCount;
        int globalCount;
        int complete;

        public ChartViewModel(RootViewModel rootViewModel)
        {
            this.rootViewModel = rootViewModel;

            Refresh();
        }

        public List<DataChart> ColorsChart
        {
            get { return colorsChart; }
        }

        public List<DataChart> RarityChart
        {
            get { return rarityChart; }
        }

        public int ExpansionsCount
        {
            get { return expansionsCount; }
        }

        public int BlocksCount
        {
            get { return blocksCount; }
        }

        public int GlobalCount
        {
            get { return globalCount; }
        }

        public string CompleteCardsCount
        {
            get
            {
                return string.Format("{0:0.00}%", ((CurrentChecked * 100.0f) / GlobalCount));
            }
        }

        public int CurrentChecked
        {
            get { return currentChecked; }
        }

        public int CurrentMissing
        {
            get { return currentMissing; }
        }

        public string OwnValueCount
        {
            get
            {
                return string.Format("{0:0.00} €", currentValueCount);
            }
        }


        public string ValueCount
        {
            get
            {
                return string.Format("{0:0.00} €", totalValueCount);
            }
        }

        public void Refresh(IEnumerable<Card> list = null)
        {
            lock (DataRoot.Entities)
            {
                Expansion expansion = rootViewModel.ExpansionViewModel.Current;
                Block block = rootViewModel.BlockViewModel.Current;

                int expansionID = -1;
                int blockID = -1;

                if (expansion != null)
                    expansionID = expansion.ID;

                if (block != null && block.Name != "Oracle")
                    blockID = block.ID;

                globalCount = GetResultCount(list, 0, 0, expansionID, blockID, null);

                // Colors
                colorsChart = new List<DataChart>();
                foreach (Color color in DataRoot.Entities.Colors)
                {
                    if (color.Name != " ")
                    {
                        int colorID = color.ID;
                        int result = GetResultCount(list, colorID, 0, expansionID, blockID, null);

                        colorsChart.Add(new DataChart {Value = result, Name = color.Name, Binding = color});
                    }
                }

                // Rarity
                rarityChart = new List<DataChart>();

                foreach (Rarity rarity in DataRoot.Entities.Rarities)
                {
                    if (rarity.Name != " ")
                    {
                        int rarityID = rarity.ID;
                        int result = GetResultCount(list, 0, rarityID, expansionID, blockID, null);

                        rarityChart.Add(new DataChart {Value = result, Name = rarity.Name});
                    }
                }

                // Expansions & blocks
                GetExpansionsAndBlocksCount(list, expansionID, blockID, out expansionsCount, out blocksCount);

                // Complete
                complete = GetResultCount(list, 0, 0, expansionID, blockID, true);

                currentValueCount = GetResultValue(list, 0, 0, expansionID, blockID, true);
                totalValueCount = GetResultValue(list, 0, 0, expansionID, blockID, null);

                currentMissing = globalCount - complete;
                currentChecked = complete;

                RaisePropertyChanged(() => GlobalCount);
                RaisePropertyChanged(() => CompleteCardsCount);
                RaisePropertyChanged(() => CurrentChecked);
                RaisePropertyChanged(() => CurrentMissing);
                RaisePropertyChanged(() => OwnValueCount);
                RaisePropertyChanged(() => ValueCount);

                RaisePropertyChanged(() => ColorsChart);
                RaisePropertyChanged(() => RarityChart);

                RaisePropertyChanged(() => ExpansionsCount);
                RaisePropertyChanged(() => BlocksCount);
            }
        }

        static void GetExpansionsAndBlocksCount(IEnumerable<Card> list, int expansionID, int blockID, out int expansionsCount, out int blocksCount)
        {
            if (list == null)
            {
                var query = from card in DataRoot.Entities.Cards
                            where (expansionID < 0 || card.ExpansionID == expansionID) && (blockID < 0 || card.Expansion.BlockID == blockID)
                            select card.ExpansionID;
                expansionsCount = query.Distinct().Count();

                var query2 = from card in DataRoot.Entities.Cards
                             where (expansionID < 0 || card.ExpansionID == expansionID) && (blockID < 0 || card.Expansion.BlockID == blockID)
                             select card.Expansion.BlockID;
                blocksCount = query2.Distinct().Count();
            }
            else
            {
                var query = from card in list
                            where (expansionID < 0 || card.ExpansionID == expansionID) && (blockID < 0 || card.Expansion.BlockID == blockID)
                            select card.ExpansionID;
                expansionsCount = query.Distinct().Count();

                var query2 = from card in list
                             where (expansionID < 0 || card.ExpansionID == expansionID) && (blockID < 0 || card.Expansion.BlockID == blockID)
                             select card.Expansion.BlockID;
                blocksCount = query2.Distinct().Count();
            }
        }

        static int GetResultCount(IEnumerable<Card> list, int colorID, int rarityID, int expansionID, int blockID, bool? check)
        {
            int result;
            if (list == null)
            {
                var query = from card in DataRoot.Entities.Cards
                            where (rarityID == 0 || card.RarityID == rarityID) && (colorID == 0 || card.ColorID == colorID) && (expansionID < 0 || card.ExpansionID == expansionID) && (blockID < 0 || card.Expansion.BlockID == blockID)
                            && (!check.HasValue || card.Check == check.Value)
                            select card;
                result = query.Count();
            }
            else
            {
                var query = from card in list
                            where (rarityID == 0 || card.RarityID == rarityID) && (colorID == 0 || card.ColorID == colorID) && (expansionID <0 || card.ExpansionID == expansionID) && (blockID < 0 || card.Expansion.BlockID == blockID)
                            && (!check.HasValue || card.Check == check.Value)
                            select card;
                result = query.Count();
            }

            return result;
        }

        static float GetResultValue(IEnumerable<Card> list, int colorID, int rarityID, int expansionID, int blockID, bool? check)
        {
            try
            {
                if (list == null)
                {
                    var query = from card in DataRoot.Entities.Cards
                                where (rarityID == 0 || card.RarityID == rarityID) && (colorID == 0 || card.ColorID == colorID) && (expansionID < 0 || card.ExpansionID == expansionID) && (blockID < 0 || card.Expansion.BlockID == blockID)
                                && (!check.HasValue || card.Check == check.Value)
                                select card.Price.HasValue ? card.Price.Value : 0;
                    return query.Count() > 0 ? (float)query.Sum() : 0;
                }
                else
                {
                    var query = from card in list
                                where (rarityID == 0 || card.RarityID == rarityID) && (colorID == 0 || card.ColorID == colorID) && (expansionID < 0 || card.ExpansionID == expansionID) && (blockID < 0 || card.Expansion.BlockID == blockID)
                                && (!check.HasValue || card.Check == check.Value)
                                select card.Price.HasValue ? card.Price.Value : 0;
                    return (float)query.Sum();
                }
            }
            catch
            {
                return 0;
            }
        }
    }
}
