﻿using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Data;
using UrzaIncubator;
using System;

namespace UrzaGatherer.ViewModels
{
    public class ExpansionViewModel : ViewModelBase
    {
        Expansion current;
        readonly Stack<IEnumerable<Card>> requestStack = new Stack<IEnumerable<Card>>();
        readonly ManualResetEvent queueEvent = new ManualResetEvent(false);
        IEnumerable<Card> currentQuery;
        Task currentQueryTask;
        CancellationTokenSource queryCancellation;
        SynchronizationContext synchronizationContext;
        CollectionViewSource cardsCollectionViewSource;
        readonly RootViewModel rootViewModel;
        string orderBy = "Number";
        ListSortDirection sortDirection = ListSortDirection.Ascending;
        bool isBusy;
        bool tooManyResults;
        bool? filterCheck;
        Color filterColor;
        Rarity filterRarity;
        Author filterAuthor;
        string filterName = "";
        string filterFlavor = "";
        string filterType = "";
        string filterFullText = "";
        string filterForceText = "";
        string filterDefenseText = "";
        string filterPriceText = "";
        string filterText = "";
        object forcedCurrent;

        List<Card> previousList;

        int currentCardID = -1;

        readonly RelayCommand resetFiltersCommand;
        readonly RelayCommand applyFiltersCommand;
        readonly RelayCommand upExpansionCommand;
        readonly RelayCommand downExpansionCommand;
        readonly RelayCommand addItemCommand;
        readonly RelayCommand deleteItemCommand;

        public event Action<int> PicturesImportationProgressChanged;
        public event Action<FileInfo[]> PicturesImportationProgressCompleted;
        public event Action<Card> PicturesImportationCardNotFound;

        public ExpansionViewModel(RootViewModel rootViewModel)
        {
            this.rootViewModel = rootViewModel;
            Task.Factory.StartNew(ProcessRequestJob, TaskCreationOptions.LongRunning);

            resetFiltersCommand = new RelayCommand(o => ResetFilters());
            upExpansionCommand = new RelayCommand(o => UpExpansion());
            downExpansionCommand = new RelayCommand(o => DownExpansion());
            addItemCommand = new RelayCommand(o => AddItem());
            deleteItemCommand = new RelayCommand(o => DeleteItem());
            applyFiltersCommand = new RelayCommand(o => Refresh());
        }

        public Expansion Current
        {
            get { return current; }
            set
            {
                if (current == value)
                    return;

                current = value;

                RaisePropertyChanged(() => Current);

                if (current != null)
                    Refresh();
            }
        }

        public CollectionViewSource Cards
        {
            get { return cardsCollectionViewSource; }
        }

        public string CardsCount
        {
            get
            {
                if (Cards == null)
                    return "";
                return string.Format("{0} card(s)", (Cards.Source as List<Card>).Count);
            }
        }

        public string OrderBy
        {
            get { return orderBy; }
            set
            {
                if (orderBy == value)
                    return;
                orderBy = value;

                ClearPrevious();
                previousList = null;
            }
        }

        public ListSortDirection SortDirection
        {
            get { return sortDirection; }
            set
            {
                if (sortDirection == value)
                    return;

                sortDirection = value;
                ClearPrevious();

                previousList = null;
            }
        }

        public bool IsBusy
        {
            get { return isBusy; }
            set
            {
                if (isBusy == value)
                    return;

                isBusy = value;

                RaisePropertyChanged(() => IsBusy);
            }
        }

        public bool TooManyResults
        {
            get { return tooManyResults; }
            set
            {
                if (tooManyResults == value)
                    return;

                tooManyResults = value;

                RaisePropertyChanged(() => TooManyResults);
            }
        }

        public Color FilterColor
        {
            get { return filterColor; }
            set
            {
                if (filterColor == value)
                    return;

                filterColor = value;
            }
        }

        public bool? FilterCheck
        {
            get { return filterCheck; }
            set
            {
                if (filterCheck == value)
                    return;

                filterCheck = value;
            }
        }

        public Rarity FilterRarity
        {
            get { return filterRarity; }
            set
            {
                if (filterRarity == value)
                    return;

                filterRarity = value;
            }
        }

        public Author FilterAuthor
        {
            get { return filterAuthor; }
            set
            {
                if (filterAuthor == value)
                    return;

                filterAuthor = value;
            }
        }

        public string FilterName
        {
            get { return filterName; }
            set
            {
                if (filterName == value)
                    return;

                filterName = value;
            }
        }

        public string FilterType
        {
            get { return filterType; }
            set
            {
                if (filterType == value)
                    return;

                filterType = value;
            }
        }

        public string FilterFlavor
        {
            get { return filterFlavor; }
            set
            {
                if (filterFlavor == value)
                    return;

                filterFlavor = value;
            }
        }

        public string FilterFullText
        {
            get { return filterFullText; }
            set
            {
                if (filterFullText == value)
                    return;

                filterFullText = value;
            }
        }

        public string FilterForce
        {
            get { return filterForceText; }
            set
            {
                if (filterForceText == value)
                    return;

                filterForceText = value;
            }
        }

        public string FilterDefense
        {
            get { return filterDefenseText; }
            set
            {
                if (filterDefenseText == value)
                    return;

                filterDefenseText = value;
            }
        }

        public string FilterPrice
        {
            get { return filterPriceText; }
            set
            {
                if (filterPriceText == value)
                    return;

                filterPriceText = value;
            }
        }

        public string FilterText
        {
            get { return filterText; }
            set
            {
                if (filterText == value)
                    return;

                filterText = value;
            }
        }

        public RelayCommand ResetFiltersCommand
        {
            get { return resetFiltersCommand; }
        }

        public RelayCommand UpExpansionCommand
        {
            get { return upExpansionCommand; }
        }

        public RelayCommand DownExpansionCommand
        {
            get { return downExpansionCommand; }
        }

        public RelayCommand AddItemCommand
        {
            get { return addItemCommand; }
        }

        public RelayCommand DeleteItemCommand
        {
            get { return deleteItemCommand; }
        }

        public RelayCommand ApplyFiltersCommand
        {
            get { return applyFiltersCommand; }
        }

        public RootViewModel RootViewModel
        {
            get { return rootViewModel; }
        }

        public void Refresh()
        {
            synchronizationContext = SynchronizationContext.Current;

            TooManyResults = false;

            if (Current == null && RootViewModel.BlockViewModel.Current == null)
                return;

            if (Current == null)
            {
                if (RootViewModel.BlockViewModel.Current.Name == "Oracle")
                {
                    DoOracleForecast();
                    return;
                }

                EnqueueQuery(null);
            }
            else
            {
                IEnumerable<Card> query = BuildQuery(Current);

                EnqueueQuery(query);
            }
        }

        void DoOracleForecast()
        {
            TooManyResults = false;

            bool dontUpdate;
            IEnumerable<Card> query = GetQuery(out dontUpdate);

            if (dontUpdate)
            {
                TooManyResults = true;
                EnqueueQuery(null);
                return;
            }

            EnqueueQuery(query);
        }

        void AddItem()
        {
            Card card = new Card
            {
                Name = "New card",
                ID = (from c in DataRoot.Entities.Cards select c.ID).Max() + 1,
                Color = DataRoot.Entities.Colors.First(),
                Author = DataRoot.Entities.Authors.First(),
                Rarity = DataRoot.Entities.Rarities.First(),
                Force = 0,
                Defense = 0,
                Expansion = Current
            };
            Current.Cards.Add(card);
            Current.Complete = false;
            Current.Block.Complete = false;

            forcedCurrent = card;

            synchronizationContext.Send(o =>
                                            {
                                                RootViewModel.Commit();
                                                RootViewModel.DialogSystem.EditCard();
                                            }, null);
            Refresh();
        }

        void DeleteItem()
        {
            Card card = Cards.View.CurrentItem as Card;
            if (card == null)
                return;
            if (RootViewModel.DialogSystem.Question("Are you sure you want to delete selected card?", "Confirmation"))
            {
                DataRoot.Entities.DeleteObject(card);

                RootViewModel.Commit();
                RootViewModel.BlockViewModel.RecomputeCounters();
                Cards.View.MoveCurrentToFirst();
            }
        }

        void UpExpansion()
        {
            Current.OrderInBlock--;

            Current.Block.RefreshSortedExpansions();
        }

        void DownExpansion()
        {
            Current.OrderInBlock++;

            Current.Block.RefreshSortedExpansions();
        }

        public void Sort()
        {
            if (cardsCollectionViewSource == null)
                return;

            cardsCollectionViewSource.SortDescriptions.Clear();
            cardsCollectionViewSource.SortDescriptions.Add(new SortDescription(OrderBy, SortDirection));
        }

        IEnumerable<Card> BuildQuery(Expansion expansion)
        {
            bool dontUpdate;
            return GetQuery(out dontUpdate, expansion.ID);
        }

        IEnumerable<Card> GetQuery(out bool dontUpdate, int expansionID = 0)
        {
            int forceFilter;
            int defenseFilter;
            float priceFilter;

            int.TryParse(FilterForce, out forceFilter);
            int.TryParse(FilterDefense, out defenseFilter);
            float.TryParse(FilterPrice, out priceFilter);

            bool enableColorFilter = FilterColor != null && FilterColor.Name != " ";
            bool enableRarityFilter = FilterRarity != null && FilterRarity.Name != " ";
            bool enableAuthorFilter = FilterAuthor != null && FilterAuthor.Name != " ";
            bool enableNameFilter = FilterName != "";
            bool enableFlavorFilter = FilterFlavor != "";
            bool enableTextFilter = FilterText != "";
            bool enableTypeFilter = FilterType != "";
            bool enableFullFilter = FilterFullText != "";

            string authorName = FilterAuthor == null ? " " : FilterAuthor.Name;
            string colorName = FilterColor == null ? " " : FilterColor.Name;
            string rarityName = FilterRarity == null ? " " : FilterRarity.Name;

            bool checkValue = false;

            if (FilterCheck.HasValue)
                checkValue = FilterCheck.Value;

            dontUpdate = (colorName == " " && rarityName == " " && authorName == " " && (FilterName.Length < 3) && (FilterText.Length < 3) && (FilterFlavor.Length < 3) && (FilterType.Length < 3) && (FilterFullText.Length < 3) && forceFilter < 2 && defenseFilter < 2 && priceFilter == 0);
           
            return from card in DataRoot.Entities.Cards
                   where
                       (expansionID <= 0 || card.ExpansionID == expansionID)
                       &&
                       (!FilterCheck.HasValue || card.Check == checkValue)
                       &&
                       (card.Color.Name == colorName || !enableColorFilter)
                       &&
                       (card.Rarity.Name == rarityName || !enableRarityFilter)
                       &&
                       (card.Author.Name == authorName || !enableAuthorFilter)
                       &&
                       (forceFilter < 2 || card.Force >= forceFilter)
                       &&
                       (defenseFilter < 2 || card.Defense >= defenseFilter)
                       &&
                       (priceFilter == 0 || card.Price >= priceFilter)
                       &&
                       (!enableNameFilter || card.Name.Contains(FilterName))
                       &&
                       (!enableTextFilter || card.Text.Contains(FilterText))
                       &&
                       (!enableFlavorFilter || card.Flavor.Contains(FilterFlavor))
                       &&
                       (!enableTypeFilter || card.Type.Contains(FilterType))
                       &&
                       (!enableFullFilter || card.Type.Contains(FilterFullText) || card.Flavor.Contains(FilterFullText) || card.Name.Contains(FilterFullText) || card.Text.Contains(FilterFullText))
                   select card;
        }

        void ProcessRequestJob()
        {
            while (true)
            {
                queueEvent.WaitOne();
                queueEvent.Reset();

                IEnumerable<Card> query = null;
                lock (requestStack)
                {
                    if (requestStack.Count > 0)
                        query = requestStack.Pop();
                    requestStack.Clear();
                }

                WaitForCurrentQueryToFinish();

                if (query == null)
                {
                    synchronizationContext.Send(o =>
                                                    {
                                                        cardsCollectionViewSource = null;
                                                        RaisePropertyChanged(() => Cards);
                                                        RaisePropertyChanged(() => CardsCount);

                                                        if (Cards != null)
                                                        {
                                                            (Cards.Source as List<Card>).ForEach(c => c.PictureFrame = null);
                                                        }
                                                    }, null);

                    RootViewModel.ChartViewModel.Refresh();
                }

                currentQuery = query;
                DispatchListBinding(query);
            }
        }

        void WaitForCurrentQueryToFinish()
        {
            if (currentQuery != null)
            {
                currentQuery = null;
                queryCancellation.Cancel();

                try
                {
                    currentQueryTask.Wait();
                }
                catch
                { }
            }
        }

        void EnqueueQuery(IEnumerable<Card> query)
        {
            if (queryCancellation != null)
                queryCancellation.Cancel();

            lock (requestStack)
            {
                requestStack.Push(query);
                queueEvent.Set();
            }
        }

        void DispatchListBinding(IEnumerable<Card> query)
        {
            synchronizationContext.Send(o =>
                                            {
                                                IsBusy = true;
                                            }, null);

            queryCancellation = new CancellationTokenSource();
            CancellationToken token = queryCancellation.Token;
            currentQueryTask = Task.Factory.StartNew(() =>
            {
                try
                {
                    lock (DataRoot.Entities)
                    {
                        if (query == null)
                            return;

                        List<Card> list = query.ToList();

                        if (token.IsCancellationRequested)
                        {
                            return;
                        }

                        bool needToRefresh = true;

                        object curr = null;

                        if (Cards != null)
                        {                 
                            needToRefresh = previousList == null || !previousList.SequenceEqual(list);

                            if (needToRefresh)
                            {
                                list.ForEach(c => c.PictureFrame = null);
                                ClearPrevious();
                                synchronizationContext.Send(o => curr = Cards.View.CurrentItem, null);
                            }
                        }

                        if (forcedCurrent != null)
                        {
                            curr = forcedCurrent;
                            forcedCurrent = null;
                        }

                        synchronizationContext.Send(o =>
                        {
                            if (needToRefresh)
                            {
                                cardsCollectionViewSource = new CollectionViewSource {Source = list};
                                cardsCollectionViewSource.SortDescriptions.Add(new SortDescription(OrderBy,
                                                                                                   SortDirection));

                                if (token.IsCancellationRequested)
                                {
                                    return;
                                }

                                if (list.Contains(curr))
                                    Cards.View.MoveCurrentTo(curr);
                                else if (Cards.View.CurrentPosition != 0)
                                    Cards.View.MoveCurrentToFirst(); 
                                
                                RaisePropertyChanged(() => Cards);
                                RaisePropertyChanged(() => CardsCount);

                                previousList = list;
                            }

                        }, null);

                        RootViewModel.ChartViewModel.Refresh(list);

                        if (needToRefresh)
                        {
                            list.ForEach(c =>
                                             {
                                                 if (token.IsCancellationRequested)
                                                 {
                                                     return;
                                                 }
                                                 c.BindPath();
                                             });
                        }
                        currentQuery = null;
                    }
                }
                catch
                {

                }

                synchronizationContext.Send(o =>
                {
                    IsBusy = false;
                }, null);
            }, token);
        }

        void ClearPrevious()
        {
            if (previousList == null)
                return;
            previousList.ForEach(c => c.PictureFrame = null);
        }

        internal void ResetFilters()
        {
            filterCheck = null;
            filterColor = null;
            filterRarity = null;
            filterAuthor = null;
            filterName = "";
            filterFlavor = "";
            filterType = "";
            filterFullText = "";
            filterForceText = "";
            filterDefenseText = "";
            filterPriceText = "";
            filterText = "";

            RaisePropertyChanged(() => FilterCheck);
            RaisePropertyChanged(() => FilterColor);
            RaisePropertyChanged(() => FilterRarity);
            RaisePropertyChanged(() => FilterAuthor);
            RaisePropertyChanged(() => FilterName);
            RaisePropertyChanged(() => FilterFlavor);
            RaisePropertyChanged(() => FilterType);
            RaisePropertyChanged(() => FilterFullText);
            RaisePropertyChanged(() => FilterForce);
            RaisePropertyChanged(() => FilterDefense);
            RaisePropertyChanged(() => FilterPrice);
            RaisePropertyChanged(() => FilterText);

            Refresh();
        }

        public void EditCurrentCard()
        {
            RootViewModel.DialogSystem.EditCard();
        }

        public void ImportPictures(string path)
        {
            Task.Factory.StartNew(() =>
            {
                DirectoryInfo info = new DirectoryInfo(path);

                int progress = 0;

                foreach (Card card in Current.Cards)
                {
                    progress++;
                    synchronizationContext.Send(o=>
                                                    {
                                                        if (PicturesImportationProgressChanged != null)
                                                            PicturesImportationProgressChanged(progress);
                                                    },  null);

                    bool find = false;
                    foreach (FileInfo file in info.GetFiles())
                    {
                        string cardName = card.Name.ToLower().Replace(")", "").Replace("(ver.", "").Replace("(v.", "").Replace("’", "'").Replace("//", "").Replace(" ", "").Replace("æ", "ae").Replace(":", "");
                        if (file.Name.Replace(file.Extension, "").ToLower().Replace(".", "").Replace(" ", "").Replace("full", "").Replace("xlhq", "") == cardName)
                        {
                            card.Image = File.ReadAllBytes(file.FullName);
                            find = true;
                            break;
                        }
                    }

                    if (!find)
                    {
                        synchronizationContext.Send(o =>
                        {
                            if (PicturesImportationCardNotFound != null)
                                PicturesImportationCardNotFound(card);
                        }, null);
                    }
                }
                synchronizationContext.Send(o =>
                {
                    if (PicturesImportationProgressCompleted != null)
                        PicturesImportationProgressCompleted(info.GetFiles());
                }, null);
            });
        }

        public Card CreateCard(Expansion expansion)
        {
            if (currentCardID == -1)
                currentCardID = (from c in DataRoot.Entities.Cards select c.ID).Max();

            Card card = new Card { ID = ++currentCardID, Expansion = expansion };

            DataRoot.Entities.Cards.AddObject(card);

            return card;
        }
    }
}