﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Controls.Primitives;
using System.Windows.Media.Animation;

namespace UrzaGatherer
{
    public enum SlideTransition
    {
        Slide,
        Fade,
    }

    [TemplatePart(Name = "PART_ItemsHost", Type = typeof(Grid))]
    public class SlideControl : Selector
    {
        private Grid _itemHost;

        public event EventHandler TransitionCompleted;

        public SlideTransition Transition
        {
            get;
            set;
        }

        /// <summary>
        /// Durée de la transition
        /// </summary>
        public Duration TransitionDuration
        {
            get;
            set;
        }

        static SlideControl()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(SlideControl), new FrameworkPropertyMetadata(typeof(SlideControl)));
            SelectedIndexProperty.OverrideMetadata(typeof(SlideControl), new FrameworkPropertyMetadata(OnSelectedIndexPropertyChanged));
        }

        private static void OnSelectedIndexPropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((SlideControl)d).UpdateTransition((int)e.OldValue, (int)e.NewValue);
        }

        public SlideControl()
        {
            TransitionDuration = new Duration(TimeSpan.FromMilliseconds(300));
        }

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            _itemHost = (Grid)GetTemplateChild("PART_ItemsHost");

            if (SelectedIndex != -1)
            {
                Dispatcher.BeginInvoke(new Action(() => UpdateTransition(-1, SelectedIndex)), System.Windows.Threading.DispatcherPriority.Loaded);
            }
        }

        protected virtual void UpdateTransition(int previousIndex, int newIndex)
        {
            if (_itemHost == null)
                return;

            if (previousIndex == newIndex)
                return;

            switch (Transition)
            {
                default:
                    SlideEffect(previousIndex, newIndex);
                    break;
                case SlideTransition.Fade:
                    FadeEffect(previousIndex, newIndex);
                    break;
            }
        }

        private void FadeEffect(int previousIndex, int newIndex)
        {
            if (previousIndex == -1)
            {
                if (newIndex != -1)
                {
                    SlideItem nextPage = (SlideItem)ItemContainerGenerator.ContainerFromIndex(newIndex);
                    ScaleTransform scale = (ScaleTransform)((TransformGroup)nextPage.RenderTransform).Children[1];
                    scale.ScaleX = 0;
                    scale.ScaleY = 0;
                    nextPage.Opacity = 1;
                    nextPage.Visibility = Visibility.Visible;
                }
            }
            else
            {
                SlideItem previousPage = previousIndex == -1 ? null : (SlideItem)ItemContainerGenerator.ContainerFromIndex(previousIndex);
                SlideItem nextPage = newIndex == -1 ? null : (SlideItem)ItemContainerGenerator.ContainerFromIndex(newIndex);

                Storyboard sb1 = FadeTransition(previousPage, TransitionDuration);
                Storyboard sb2 = UnfadeTransition(nextPage, TransitionDuration);

                if (sb1 != null)
                    sb1.Begin();

                if (sb2 != null)
                {
                    sb2.BeginTime = TransitionDuration.TimeSpan;
                    sb2.Begin();
                }
            }
        }

        private static Storyboard UnfadeTransition(SlideItem page, Duration duration)
        {
            Storyboard sb = new Storyboard();
            sb.Children.Add(AnimationUtils.CreateDoubleAnimation(page, 1.0, new PropertyPath("(UIElement.Opacity)"), duration, new CircleEase { EasingMode = EasingMode.EaseOut }));
            sb.Children.Add(AnimationUtils.CreateVisibilityAnimation(page, TimeSpan.Zero, Visibility.Visible));
            sb.Children.Add(AnimationUtils.CreateDoubleAnimation(page, 1.0, new PropertyPath("(UIElement.RenderTransform).(TransformGroup.Children)[1].(ScaleTransform.ScaleX)"), duration, new CircleEase { EasingMode = EasingMode.EaseOut }));
            sb.Children.Add(AnimationUtils.CreateDoubleAnimation(page, 1.0, new PropertyPath("(UIElement.RenderTransform).(TransformGroup.Children)[1].(ScaleTransform.ScaleY)"), duration, new CircleEase { EasingMode = EasingMode.EaseOut }));
            return sb;
        }

        private static Storyboard FadeTransition(SlideItem page, Duration duration)
        {
            Storyboard sb = new Storyboard();
            sb.Children.Add(AnimationUtils.CreateDoubleAnimation(page, 0.0, new PropertyPath("(UIElement.Opacity)"), duration, new CircleEase { EasingMode = EasingMode.EaseOut }));
            sb.Children.Add(AnimationUtils.CreateVisibilityAnimation(page, duration.TimeSpan, Visibility.Hidden));
            sb.Children.Add(AnimationUtils.CreateDoubleAnimation(page, 0.5, new PropertyPath("(UIElement.RenderTransform).(TransformGroup.Children)[1].(ScaleTransform.ScaleX)"), duration, new CircleEase { EasingMode = EasingMode.EaseOut }));
            sb.Children.Add(AnimationUtils.CreateDoubleAnimation(page, 0.5, new PropertyPath("(UIElement.RenderTransform).(TransformGroup.Children)[1].(ScaleTransform.ScaleY)"), duration, new CircleEase { EasingMode = EasingMode.EaseOut }));
            return sb;
        }

        private void SlideEffect(int previousIndex, int newIndex)
        {
            if (previousIndex == -1)
            {
                if (newIndex != -1)
                {
                    SlideItem nextPage = (SlideItem)ItemContainerGenerator.ContainerFromIndex(newIndex);
                    TranslateTransform translate = (TranslateTransform)((TransformGroup)nextPage.RenderTransform).Children[0];
                    translate.X = 0;
                    translate.Y = 0;
                    nextPage.Visibility = Visibility.Visible;
                }
            }
            else
            {
                SlideItem previousPage = previousIndex == -1 ? null : (SlideItem)ItemContainerGenerator.ContainerFromIndex(previousIndex);
                SlideItem nextPage = newIndex == -1 ? null : (SlideItem)ItemContainerGenerator.ContainerFromIndex(newIndex);

                double actualWidth = _itemHost.ActualWidth;

                if (newIndex > previousIndex)
                {
                    Storyboard sb1 = TranslateSlideItem(previousPage, 0, -actualWidth, Visibility.Collapsed);
                    Storyboard sb2 = TranslateSlideItem(nextPage, actualWidth, 0, Visibility.Visible);

                    if (sb1 != null)
                        sb1.Begin();

                    if (sb2 != null)
                        sb2.Begin();
                }
                else
                {
                    Storyboard sb1 = TranslateSlideItem(previousPage, 0, actualWidth, Visibility.Collapsed);
                    Storyboard sb2 = TranslateSlideItem(nextPage, -actualWidth, 0, Visibility.Visible);

                    if (sb1 != null)
                        sb1.Begin();

                    if (sb2 != null)
                        sb2.Begin();
                }
            }
        }

        private Storyboard TranslateSlideItem(SlideItem pageItem, double fromX, double toX, Visibility visibility)
        {
            if (pageItem == null)
                return null;

            Storyboard sb = new Storyboard();

            DoubleAnimationUsingKeyFrames animationX = new DoubleAnimationUsingKeyFrames { Duration = TransitionDuration };
            Storyboard.SetTarget(animationX, pageItem);
            Storyboard.SetTargetProperty(animationX, new PropertyPath("(UIElement.RenderTransform).(TransformGroup.Children)[0].(TranslateTransform.X)"));

            EasingDoubleKeyFrame kf1 = new EasingDoubleKeyFrame
                                           {
                                               KeyTime = KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(0)),
                                               Value = fromX,
                                               EasingFunction = new CircleEase { EasingMode = EasingMode.EaseOut }
                                           };
            animationX.KeyFrames.Add(kf1);

            kf1 = new EasingDoubleKeyFrame
                      {
                          KeyTime = KeyTime.FromTimeSpan(TransitionDuration.TimeSpan),
                          Value = toX,
                          EasingFunction = new CircleEase { EasingMode = EasingMode.EaseOut }
                      };
            animationX.KeyFrames.Add(kf1);
            sb.Children.Add(animationX);

            sb.Children.Add(AnimationUtils.CreateVisibilityAnimation(pageItem,
                visibility == Visibility.Visible ? TimeSpan.Zero : TransitionDuration.TimeSpan, visibility));
            sb.Completed += sb_Completed;
            return sb;
        }

        void sb_Completed(object sender, EventArgs e)
        {
            if (TransitionCompleted != null)
                TransitionCompleted(this, EventArgs.Empty);
        }

        protected override DependencyObject GetContainerForItemOverride()
        {
            return new SlideItem();
        }

        protected override bool IsItemItsOwnContainerOverride(object item)
        {
            return item is SlideItem;
        }


        #region Navigation

        public void GoForward()
        {
            if (CanGoForward)
                SelectedIndex++;
        }

        public void GoBackward()
        {
            if (CanGoBackward)
                SelectedIndex--;
        }

        public bool CanGoForward
        {
            get
            {
                return SelectedIndex < Items.Count - 1;
            }
        }

        public bool CanGoBackward
        {
            get
            {
                return (SelectedIndex != 0);
            }
        }

        #endregion Navigation

    }
}
