﻿using System;
using System.Windows.Controls;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Controls.Primitives;

namespace UrzaGatherer
{

    [TemplatePart(Name = "PART_ScrollViewer", Type = typeof(ScrollViewer))]
    [TemplatePart(Name = "PART_ScrollThumb", Type = typeof(Thumb))]
    [TemplatePart(Name = "PART_SelectionMagnifier", Type = typeof(FrameworkElement))]
    public class WizardHeader : ListBox
    {
        FrameworkElement _selectionMagnifier;
        ScrollViewer _scrollViewer;
        Thumb _scrollThumb;

        public Duration TransitionDuration
        {
            get;
            set;
        }


        static WizardHeader()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(WizardHeader), new FrameworkPropertyMetadata(typeof(WizardHeader)));
            SelectedIndexProperty.OverrideMetadata(typeof(WizardHeader), new FrameworkPropertyMetadata(OnSelectedIndexPropertyChanged));
        }

        protected override void OnSelectionChanged(SelectionChangedEventArgs e)
        {
            SelectedIndex = SelectedIndex;
            base.OnSelectionChanged(e);
        }

        private static void OnSelectedIndexPropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((WizardHeader)d).UpdateMagnifier((int)e.OldValue, (int)e.NewValue);
        }

        public WizardHeader()
        {
            TransitionDuration = new Duration(TimeSpan.FromMilliseconds(300));

        }

        protected override Size ArrangeOverride(Size arrangeBounds)
        {
            Size s = base.ArrangeOverride(arrangeBounds);
            UpdateMagnifier(-1, SelectedIndex);
            return s;
        }

        private void UpdateMagnifier(int oldItem, int newItem)
        {
            if (_selectionMagnifier == null)
                return;

            try
            {
                InternalUpdateMagnifier(oldItem, newItem);
            }
            catch
            { }
        }

        private void InternalUpdateMagnifier(int oldItem, int newItem)
        {

            FrameworkElement oldListViewItem = null;

            if (oldItem != -1)
            {
                oldListViewItem = (FrameworkElement)ItemContainerGenerator.ContainerFromIndex(oldItem);
            }

            if (newItem != -1)
            {
                FrameworkElement newListViewItem = (FrameworkElement)ItemContainerGenerator.ContainerFromIndex(newItem);

                _selectionMagnifier.Opacity = 1;

                Vector position = VisualTreeHelper.GetOffset(newListViewItem);

                if (oldListViewItem != null)
                {
                    Storyboard sb = new Storyboard();

                    sb.Children.Add(AnimationUtils.CreateDoubleAnimation(_selectionMagnifier,
                                                            position.X,
                                                            new PropertyPath("(FrameworkElement.RenderTransform).(TranslateTransform.X)"),
                                                            TransitionDuration,
                                                            new CircleEase { EasingMode = EasingMode.EaseOut }));

                    sb.Children.Add(AnimationUtils.CreateDoubleAnimation(_selectionMagnifier,
                                                            position.Y,
                                                            new PropertyPath("(FrameworkElement.RenderTransform).(TranslateTransform.Y)"),
                                                            TransitionDuration,
                                                            new CircleEase { EasingMode = EasingMode.EaseOut }));

                    sb.Children.Add(AnimationUtils.CreateDoubleAnimation(_selectionMagnifier,
                                                            newListViewItem.ActualWidth,
                                                            new PropertyPath("(FrameworkElement.Width)"),
                                                            TransitionDuration,
                                                            new CircleEase { EasingMode = EasingMode.EaseOut }));

                    sb.Children.Add(AnimationUtils.CreateDoubleAnimation(_selectionMagnifier,
                                                           newListViewItem.ActualHeight,
                                                           new PropertyPath("(FrameworkElement.Height)"),
                                                           TransitionDuration,
                                                           new CircleEase { EasingMode = EasingMode.EaseOut }));
                    sb.Begin();
                }
                else
                {
                    TranslateTransform translate = _selectionMagnifier.RenderTransform as TranslateTransform;

                    _selectionMagnifier.Width = newListViewItem.ActualWidth;
                    _selectionMagnifier.Height = newListViewItem.ActualHeight;
                    translate.X = position.X;
                    translate.Y = position.Y;

                    _selectionMagnifier.SetCurrentValue(WidthProperty, newListViewItem.ActualWidth);
                    _selectionMagnifier.SetCurrentValue(HeightProperty, newListViewItem.ActualHeight);
                    translate.SetCurrentValue(TranslateTransform.XProperty, position.X);
                    translate.SetCurrentValue(TranslateTransform.YProperty, position.Y);
                }
            }
            else
            {
                _selectionMagnifier.Opacity = 0;
            }
        }
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            _scrollThumb = GetTemplateChild("PART_ScrollThumb") as Thumb;
            _scrollViewer = GetTemplateChild("PART_ScrollViewer") as ScrollViewer;

            if (_scrollThumb != null && _scrollViewer != null)
            {
                _scrollThumb.DragStarted += _scrollThumb_DragStarted;
                _scrollThumb.DragDelta += _scrollThumb_DragDelta;
            }

            _selectionMagnifier = GetTemplateChild("PART_SelectionMagnifier") as FrameworkElement;
            _selectionMagnifier.RenderTransformOrigin = new Point(0, 0);
            _selectionMagnifier.RenderTransform = new TranslateTransform();
            _selectionMagnifier.HorizontalAlignment = HorizontalAlignment.Left;
            _selectionMagnifier.VerticalAlignment = VerticalAlignment.Top;

            if (SelectedIndex != -1)
            {
                Dispatcher.BeginInvoke(new Action(() => UpdateMagnifier(-1, SelectedIndex)), System.Windows.Threading.DispatcherPriority.Loaded);
            }
        }

        private double _horizontalOffset;

        void _scrollThumb_DragStarted(object sender, DragStartedEventArgs e)
        {
            _horizontalOffset = _scrollViewer.HorizontalOffset;
        }

        void _scrollThumb_DragDelta(object sender, DragDeltaEventArgs e)
        {
            _scrollViewer.ScrollToHorizontalOffset(_horizontalOffset + e.HorizontalChange);
        }


    }
}
