﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Threading;
using mshtml;
using UrzaIncubator;

namespace UrzaGatherer
{
    /// <summary>
    /// Interaction logic for ExpansionImporter.xaml
    /// </summary>
    public partial class ExpansionImporter
    {
        UrzaGathererEntities entities;
        Block parentBlock;
        bool cancelRequested;
        int currentAuthorID;
        int currentCardID;

        public UrzaGathererEntities Entities
        {
            set { entities = value; }
        }

        public Block ParentBlock
        {
            set { parentBlock = value; }
        }

        public ExpansionImporter()
        {
            InitializeComponent();
        }

        Author GetAuthorFromName(string authorName)
        {
            var queryAuthor = from authordb in entities.Authors
                              where string.Compare(authordb.Name, authorName, true) == 0
                              select authordb;

            if (queryAuthor.Count() > 0)
            {
                return queryAuthor.First();
            }
            Author newAuthor = new Author();
            newAuthor.ID = ++currentAuthorID;
            newAuthor.Name = authorName;

            entities.AddToAuthors(newAuthor);
            return newAuthor;
        }

        Color GetColorFromName(string colorName)
        {
            if (colorName == "Colorless")
                colorName = "Artifact";
            else if (string.IsNullOrEmpty(colorName))
                colorName = "Artifact";
            else if (colorName.Contains('/'))
                colorName = "Multicolor";

            var query = from color in entities.Colors
                        where string.Compare(color.Name, colorName, true) == 0
                        select color;

            return query.First();
        }

        static int GetRarityFromName(string rarityName)
        {
            switch (rarityName.Substring(0, 1))
            {
                case "M":
                    return 4;
                case "R":
                    return 3;
                case "U":
                    return 2;
                default:
                    return 1;
            }
        }

        void ImportExpansion()
        {
            bool importPictures = ImportFromMagicCardsInfo.IsChecked == true;
            entities.SaveChanges();
            LogText.Text = "Importation started.\n\n";

            Expansion expansion = null;
            string expansionName = ExpansionName.Text;

            if (string.IsNullOrEmpty(expansionName))
            {
                LogText.Text += "Invalid expansion name.";
                return;
            }

            Task.Factory.StartNew(() =>
             {
                 try
                 {
                     currentAuthorID = (from a in entities.Authors select a.ID).Max();
                     currentCardID = (from c in entities.Cards select c.ID).Max();
                     string url = string.Format("http://gatherer.wizards.com/Pages/Search/Default.aspx?output=checklist&set=[\"{0}\"]&special=true", expansionName);

                     string data = Tools.GetData(url);
                     IHTMLDocument2 document = (IHTMLDocument2)new HTMLDocument();
                     document.write(new object[] { data.Replace(@"<?xml version=""1.0"" encoding=""utf-8"" ?>", "") });
                     document.close();
                     List<IHTMLElement> elements = document.body.GetElementsByClass("cardItem");

                     if (elements.Count == 0)
                     {
                         Dispatcher.Invoke(DispatcherPriority.Render, new Action(() =>
                           {
                               LogText.Text += "Expansion not found.";
                           }));
                         return;
                     }

                     Dispatcher.Invoke(DispatcherPriority.Render, new Action(() =>
                        {
                            ImportationProgress.Maximum = elements.Count - 1;
                            ImportationProgress.Value = 0;

                            expansion = new Expansion();
                            expansion.ID = (from e in entities.Expansions select e.ID).Max() + 1;
                            expansion.Name = expansionName;
                            expansion.Block = parentBlock;
                            expansion.FrenchVersion = true;

                            parentBlock.Expansions.Add(expansion);

                            entities.AddToExpansions(expansion);
                        }));

                     bool numberNotFound = false;
                     int localNumber = 1;
                     foreach (IHTMLElement element in elements)
                     {
                         if (cancelRequested)
                             return;
                         Card card = new Card();
                         card.ID = ++currentCardID;
                         card.Expansion = expansion;

                         if (!numberNotFound)
                         {
                             try
                             {
                                 card.Number = int.Parse(element.GetElementTextByClass("number"));
                             }
                             catch
                             {
                                 numberNotFound = true;
                             }
                         }

                         if (numberNotFound)
                             card.Number = localNumber++;

                         card.Name = element.GetElementTextByClass("name");
                         card.Author = GetAuthorFromName(element.GetElementTextByClass("artist"));
                         card.Color = GetColorFromName(element.GetElementTextByClass("color"));
                         card.RarityID = GetRarityFromName(element.GetElementTextByClass("rarity"));

                         entities.Cards.AddObject(card);
                         if (importPictures)
                             Tools.GetCardImage2(card, "en");
                         Tools.GetCardInfos(card);

                         Dispatcher.Invoke(DispatcherPriority.Render, new Action(() =>
                         {
                             ImportationProgress.Value++;
                             ImportationInfo.Text = string.Format("Importing {0} - {1:0.00}%",
                                                                  card.Name,
                                                                  ((ImportationProgress.Value + 1) * 100) /
                                                                  elements.Count);
                             LogText.Text += string.Format("[{3:000}] {0} - {1} - {2}\n",
                                                           card.Name, card.Color.Name,
                                                           card.Author.Name, card.Number);
                             LogText.ScrollToEnd();
                         }));
                     }

                     entities.SaveChanges();
                     Dispatcher.Invoke(DispatcherPriority.Render, new Action(() =>
                     {
                         CancelButton.Visibility = Visibility.Collapsed;
                         AcceptButton.Visibility = Visibility.Visible;
                         LogText.Text += "\n\nImportation done.";
                         LogText.ScrollToEnd();
                         ImportationInfo.Text = "Importation done";
                     }));
                 }
                 catch (Exception ex)
                 {
                     if (cancelRequested)
                         return;
                     MessageWindow.Show("Error : " + ex.Message, "Urza Gatherer", this);
                     Cancel();
                 }
             });
        }        

        private void Cancel()
        {
            cancelRequested = true;
            foreach (var entity in entities.ObjectStateManager.GetObjectStateEntries(EntityState.Added))
            {
                entity.ChangeState(EntityState.Detached);
            }
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            Cancel();
            DialogResult = false;
        }

        private void Launch_Click(object sender, RoutedEventArgs e)
        {
            CancelButton.IsEnabled = true;
            Launch.IsEnabled = false;
            ExpansionName.IsEnabled = false;
            ImportFromMagicCardsInfo.IsEnabled = false;
            ImportExpansion();
        }

        private void AcceptButton_Click(object sender, RoutedEventArgs e)
        {
            DialogResult = true;
        }
    }
}
