﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml;
using Ookii.Dialogs.Wpf;
using UrzaGatherer.ViewModels;
using UrzaIncubator;
using Application = System.Windows.Application;
using MessageBox = System.Windows.MessageBox;

namespace UrzaGatherer
{
    public partial class MainWindow
    {
        readonly ManualResetEvent resetEvent = new ManualResetEvent(false);
        
        readonly Queue<Card> imagesSearchingCards = new Queue<Card>();

        RootViewModel rootViewModel;

        public MainWindow()
        {
            InitializeComponent();
            Title = "UrzaGatherer v" + Assembly.GetExecutingAssembly().GetName().Version;

            RootViewModel.DialogSystem.RegisterQueryFunc((title, question) =>
            {
                QueryWindow query = new QueryWindow
                {
                    Owner = this,
                    Question = "Block Name:",
                    Text = "",
                    Title = "Add a new block"
                };

                if (query.ShowDialog() == true)
                {
                    return query.Text;
                }

                return null;
            });

            RootViewModel.DialogSystem.RegisterQuestionFunc((question, title) => QuestionWindow.Show(question, title, this) == true);
            RootViewModel.DialogSystem.RegisterEditCardAction(CardEditor.Show);
            RootViewModel.DialogSystem.RegisterShowMessageAction((message, title) => MessageWindow.Show(message, title, this));
            RootViewModel.DialogSystem.RegisterShowWaitCursorAction(wait => Cursor = (wait ? Cursors.Wait : Cursors.Arrow));
            
            string path = Path.Combine(System.Windows.Forms.Application.StartupPath, "UrzaGatherer.xml");

            MainHeader.DataContext = new[] { "Home", "Tools", "Database" };
            RightHeader.DataContext = new[] { "Images", "Statistics", "Filters" };

            if (File.Exists(path))
            {
                XmlTextReader reader = new XmlTextReader(path);

                reader.ReadToFollowing("config");
                WindowState = (WindowState)Enum.Parse(typeof(WindowState), reader.GetAttribute("windowState"));

                ScaleSlider.Value = double.Parse(reader.GetAttribute("ImageScale"));

                if (reader.GetAttribute("splitter3") != null)
                {
                    MainColumn01.Width = new GridLength(double.Parse(reader.GetAttribute("splitter1")), GridUnitType.Star);
                    MainColumn02.Width = new GridLength(double.Parse(reader.GetAttribute("splitter2")), GridUnitType.Star);
                    MainColumn03.Width = new GridLength(double.Parse(reader.GetAttribute("splitter3")), GridUnitType.Star);
                }

                reader.Close();
            }

            Task.Factory.StartNew(ThreadJob, TaskCreationOptions.LongRunning);

            ImagesSlideControl.TransitionCompleted += ImagesSlideControl_TransitionCompleted;
        }

        void ImagesSlideControl_TransitionCompleted(object sender, EventArgs e)
        {
            picturesRootGrid.DelayedScrollToCurrent();
        }

        void ThreadJob()
        {
            do
            {
                resetEvent.WaitOne();

                while (imagesSearchingCards.Count > 0)
                {
                    Card card;
                    lock (imagesSearchingCards)
                    {
                        card = imagesSearchingCards.Dequeue();
                        resetEvent.Reset();
                    }

                    if (card.CompletePath == null)
                    {
                        if (!Tools.GetCardImage2(card, "en"))
                        {
                            Tools.GetCardImage2(card, "en", "a");
                        }

                    }
                    if (card.Type == null || card.Text == null)
                        Tools.GetCardInfos(card);
                }

            }
            while (true);
        }

        private void CommandBinding_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            Application.Current.Shutdown();
        }

        private void Window_Closing(object sender, CancelEventArgs e)
        {
            bool canLeave;
            do
            {
                try
                {
                    rootViewModel.Commit();
                    canLeave = true;
                }
                catch (Exception)
                {
                    canLeave = false;
                    Thread.Sleep(100);
                }
            }
            while (!canLeave);


            XmlTextWriter writer = new XmlTextWriter(Path.Combine(System.Windows.Forms.Application.StartupPath, "UrzaGatherer.xml"), null);

            writer.WriteStartDocument();
            writer.WriteStartElement("config");

            writer.WriteStartAttribute("ImageScale");
            writer.WriteValue((int)ScaleSlider.Value);
            writer.WriteEndAttribute();

            writer.WriteStartAttribute("splitter1");
            writer.WriteValue(MainColumn01.ActualWidth);
            writer.WriteEndAttribute();

            writer.WriteStartAttribute("splitter2");
            writer.WriteValue(MainColumn02.ActualWidth);
            writer.WriteEndAttribute();

            writer.WriteStartAttribute("splitter3");
            writer.WriteValue(MainColumn03.ActualWidth);
            writer.WriteEndAttribute();

            writer.WriteStartAttribute("windowState");
            writer.WriteValue(WindowState.ToString());
            writer.WriteEndAttribute();

            writer.WriteEndElement();
            writer.WriteEndDocument();

            writer.Close();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            DoMainBinding();
        }

        private void DoMainBinding()
        {
            rootViewModel = (Application.Current as App).RootViewModel;
            DataContext = rootViewModel;
        }

        private void AddExpansion_Click(object sender, RoutedEventArgs e)
        {
            ExpansionImporter importer = new ExpansionImporter();
            if (importer.ShowDialog() == true)
            {
                rootViewModel.BlockViewModel.Refresh();
            }
        }

        private void AffectHDPictures_Click(object sender, RoutedEventArgs e)
        {
            ImportPicturesWindow wnd = new ImportPicturesWindow();

            wnd.ShowDialog();
        }

        private void BackupButton_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                VistaSaveFileDialog saveDialog = new VistaSaveFileDialog
                                                     {
                                                         Filter = @"Backup files|*.bak",
                                                         Title = @"Please select a filename",
                                                         AddExtension = true,
                                                         DefaultExt = ".bak"
                                                     };
                if (saveDialog.ShowDialog(this) == true)
                {
                    Cursor = Cursors.Wait;
                    Utilities.DoEvents();

                    RootViewModel.Backup(saveDialog.FileName);

                    Cursor = Cursors.Arrow;
                    MessageWindow.Show("Backup successfull.", "UrzaGatherer", this);
                }
            }
            catch (Exception ex)
            {
                Cursor = Cursors.Arrow;
                MessageBox.Show(ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void RestoreButton_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                VistaOpenFileDialog openDialog = new VistaOpenFileDialog
                                                     {
                                                         Filter = @"All files|*.*",
                                                         Title = @"Please select a filename"
                                                     };
                if (openDialog.ShowDialog(this) == true)
                {
                    ProcessStartInfo info = new ProcessStartInfo(Path.Combine(System.Windows.Forms.Application.StartupPath, "DatabaseRestore.exe"), openDialog.FileName);
                    Process.Start(info);
                    Application.Current.Shutdown();
                }
            }
            catch (Exception ex)
            {
                Cursor = Cursors.Arrow;
                MessageWindow.Show(ex.Message, "Error", this);
            }
        }

        private void picturesRootGrid_SelectedCardChanged(Card card)
        {
            if (card.Bitmap.PixelWidth > card.Bitmap.PixelHeight)
            {
                FullImage.Stretch = Stretch.Uniform;
                FullImageTab.Height = 728 + 12;
                FullImageTab.Width = 1043 + 18;
            }
            else
            {
                FullImage.Stretch = Stretch.None;
                FullImageTab.Height = 680 + 12;
                FullImageTab.Width = 480 + 24;
            }
        }

        private void picturesRootGrid_CardWithoutPictureDetected(Card card)
        {
            lock (imagesSearchingCards)
            {
                imagesSearchingCards.Enqueue(card);
                resetEvent.Set();
            }
        }

        private void generateJSON_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                Cursor = Cursors.Wait;
                Utilities.DoEvents();

                Tools.GenerateJSON();
                Tools.GenerateMinifiedJSON();
                
                Cursor = Cursors.Arrow;
                MessageWindow.Show("Generation successfull.", "UrzaGatherer", this);
            }
            catch (Exception ex)
            {
                Cursor = Cursors.Arrow;
                MessageWindow.Show(ex.Message, "Error", this);
            }
        }

        //private void blockTreeview_SelectedItemChanged(object sender, EventArgs e)
        //{
        //    RefreshCardsGrid();
        //    //TODO if (SortCursor.Opacity == 0)
        //    //    MoveCursorToTextBlock(numberColumHeader);
        //}
    }
}
